import { Construct } from '@aws-cdk/core';
import lambda = require('@aws-cdk/aws-lambda');
import dynamodb = require('@aws-cdk/aws-dynamodb');
import cloudwatch = require('@aws-cdk/aws-cloudwatch');
import { WatchDynamoTableOptions, WatchDynamoTable } from './dynamodb';
import { IWatchful, SectionOptions } from './api';
import { WatchLambdaFunctionOptions, WatchLambdaFunction } from './lambda';
export interface WatchfulProps {
    readonly alarmEmail?: string;
}
export declare class Watchful extends Construct implements IWatchful {
    static isWatchable(obj: any): obj is IWatchable;
    private readonly dash;
    private readonly alarmTopic?;
    constructor(scope: Construct, id: string, props?: WatchfulProps);
    addWidgets(...widgets: cloudwatch.IWidget[]): void;
    addAlarm(alarm: cloudwatch.Alarm): void;
    addSection(title: string, options?: SectionOptions): void;
    watch(title: string, obj: IWatchable): void;
    watchDynamoTable(title: string, table: dynamodb.Table, options?: WatchDynamoTableOptions): WatchDynamoTable;
    watchLambdaFunction(title: string, fn: lambda.Function, options?: WatchLambdaFunctionOptions): WatchLambdaFunction;
}
export interface IWatchable {
    addToWatchful(watchful: IWatchful, title: string): void;
}
